# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from oslo_config import cfg
from oslo_utils import importutils

from kuryr.lib import exceptions


def _verify_driver(driver):
    if driver.__name__ not in cfg.CONF.binding.enabled_drivers:
        raise exceptions.DriverNotEnabledException(
            'Driver %s is not enabled' % driver.__name__)


def port_bind(endpoint_id, port, subnets, network=None, vm_port=None,
              segmentation_id=None, driver=None, **kwargs):
    """Binds the Neutron port to the network interface on the host.

    :param endpoint_id:   the ID of the endpoint as string
    :param port:         the container Neutron port dictionary as returned by
                         python-neutronclient
    :param subnets:      an iterable of all the Neutron subnets which the
                         endpoint is trying to join
    :param network:      the Neutron network which the endpoint is trying to
                         join
    :param vm_port:      the Nova instance port dictionary, as returned by
                         python-neutronclient. Binding is being done for the
                         port of a container which is running inside this Nova
                         instance (either ipvlan/macvlan or a subport).
    :param segmentation_id: ID of the segment for container traffic isolation)
    :param driver:       the binding driver name
    :param kwargs:       Additional driver-specific arguments
    :returns: the tuple of the names of the veth pair and the tuple of stdout
              and stderr returned by processutils.execute invoked with the
              executable script for binding
    :raises: kuryr.common.exceptions.VethCreationFailure,
             kuryr.common.exceptions.DriverNotEnabledException,
             processutils.ProcessExecutionError
    """
    driver = importutils.import_module(
        driver or cfg.CONF.binding.default_driver)
    _verify_driver(driver)

    return driver.port_bind(endpoint_id, port, subnets, network=network,
                            vm_port=vm_port,
                            segmentation_id=segmentation_id,
                            **kwargs)


def port_unbind(endpoint_id, neutron_port, driver=None, **kwargs):
    """Unbinds the Neutron port from the network interface on the host.

    :param endpoint_id: the ID of the Docker container as string
    :param neutron_port: a port dictionary returned from python-neutronclient
    :param driver:       the binding driver name
    :param kwargs:       Additional driver-specific arguments
    :returns: the tuple of stdout and stderr returned by processutils.execute
              invoked with the executable script for unbinding
    :raises: processutils.ProcessExecutionError, pyroute2.NetlinkError,
             kuryr.common.exceptions.DriverNotEnabledException,
    """
    driver = importutils.import_module(
        driver or cfg.CONF.binding.default_driver)
    _verify_driver(driver)

    return driver.port_unbind(endpoint_id, neutron_port, **kwargs)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""It only supports container-in-vm deployments"""

from kuryr.lib.binding.drivers import nested
from kuryr.lib.binding.drivers import utils

KIND = 'vlan'


def port_bind(endpoint_id, port, subnets, network=None,
              vm_port=None, segmentation_id=None, **kwargs):
    """Binds the Neutron port to the network interface on the host.

    :param endpoint_id:   the ID of the endpoint as string
    :param port:         the container Neutron port dictionary as returned by
                         python-neutronclient
    :param subnets:      an iterable of all the Neutron subnets which the
                         endpoint is trying to join
    :param network:      the Neutron network which the endpoint is trying to
                         join
    :param vm_port:      the Nova instance port dictionary, as returned by
                         python-neutronclient. Container is running inside this
                         instance (either ipvlan/macvlan or a subport)
    :param segmentation_id: ID of the segment for container traffic isolation)
    :param kwargs:       Additional driver-specific arguments
    :returns: the tuple of the names of the veth pair and the tuple of stdout
              and stderr returned by processutils.execute invoked with the
              executable script for binding
    :raises: kuryr.common.exceptions.VethCreationFailure,
             processutils.ProcessExecutionError
    """
    ip = utils.get_ipdb()
    port_id = port['id']
    _, devname = utils.get_veth_pair_names(port_id)
    link_iface = nested.get_link_iface(vm_port)
    with ip.create(ifname=devname, kind=KIND,
                   link=ip.interfaces[link_iface],
                   address=port.get(utils.MAC_ADDRESS_KEY),
                   vlan_id=segmentation_id) as container_iface:
        utils._configure_container_iface(
            container_iface, subnets,
            fixed_ips=port.get(utils.FIXED_IP_KEY))

    return None, devname, ('', None)


port_unbind = nested.port_unbind



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from kuryr.lib import opts as kuryr_opts
from kuryr.tests.unit import base


class OptsTest(base.TestCase):

    _fake_kuryr_opts = [(None, 'fakevalue1'), ('Key1', 'fakevalue2')]
    _fake_neutron_opts = [('poolv4', 'swimming4'), ('poolv6', 'swimming6')]
    _fake_binding_group = 'binding_group'
    _fake_binding_opts = [('driver', 'my.ipvlan')]

    @mock.patch.multiple(kuryr_opts.config,
                         binding_group=_fake_binding_group,
                         binding_opts=_fake_binding_opts)
    @mock.patch.multiple(kuryr_opts,
                         _kuryr_opts=_fake_kuryr_opts,
                         list_neutron_opts=mock.DEFAULT)
    def test_list_kuryr_opts(self, list_neutron_opts):
        list_neutron_opts.return_value = self._fake_neutron_opts

        opts = self._fake_kuryr_opts + self._fake_neutron_opts + [(
            self._fake_binding_group, self._fake_binding_opts)]
        self.assertEqual(opts, kuryr_opts.list_kuryr_opts())



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

import ddt
import socket

from oslo_config import cfg

from kuryr.lib import config as kuryr_config
from kuryr.lib import utils
from kuryr.tests.unit import base


@ddt.ddt
class TestKuryrUtils(base.TestCase):
    """Unit tests for utilities."""
    def setUp(self):
        super(TestKuryrUtils, self).setUp()
        self.fake_url = 'http://127.0.0.1:9696'
        self.fake_auth_url = 'http://127.0.0.1:5000'

    def test_get_subnetpool_name(self):
        fake_subnet_cidr = "10.0.0.0/16"
        generated_neutron_subnetpool_name = utils.get_neutron_subnetpool_name(
            fake_subnet_cidr)
        name_prefix = cfg.CONF.subnetpool_name_prefix
        self.assertIn(name_prefix, generated_neutron_subnetpool_name)
        self.assertIn(fake_subnet_cidr, generated_neutron_subnetpool_name)

    @mock.patch('keystoneauth1.loading.load_auth_from_conf_options')
    def test_get_auth_plugin(self, mock_auth_loader):
        conf_group = 'neutron'
        fake_auth = 'Fake_auth_plugin'
        mock_auth_loader.return_value = fake_auth
        utils.get_auth_plugin(conf_group)
        mock_auth_loader.assert_called_once_with(cfg.CONF, conf_group)

    @mock.patch('keystoneauth1.loading.load_session_from_conf_options')
    def test_get_keystone_session(self, mock_session_loader):
        conf_group = 'neutron'
        fake_auth = 'Fake_auth_plugin'
        fake_session = 'Fake_session_plugin'
        mock_session_loader.return_value = fake_session
        utils.get_keystone_session(conf_group, fake_auth)
        mock_session_loader.assert_called_once_with(cfg.CONF, conf_group,
                                                    auth=fake_auth)

    @mock.patch('neutronclient.v2_0.client.Client')
    @mock.patch('kuryr.lib.utils.get_auth_plugin')
    @mock.patch('kuryr.lib.utils.get_keystone_session')
    def test_get_neutron_client(self, mock_get_keystone_session,
                                mock_get_auth_plugin, mock_client):
        fake_auth = 'Fake_auth_plugin'
        fake_session = 'Fake_session_plugin'
        default_conf_group = 'neutron'
        mock_get_auth_plugin.return_value = fake_auth
        mock_get_keystone_session.return_value = fake_session
        utils.get_neutron_client()
        mock_get_keystone_session.assert_called_once_with(default_conf_group,
                                                          fake_auth)
        mock_get_auth_plugin.assert_called_once_with(default_conf_group)
        neutron_group = getattr(cfg.CONF, kuryr_config.neutron_group.name)
        mock_client.assert_called_once_with(
            auth=fake_auth,
            session=fake_session,
            endpoint_type=neutron_group.endpoint_type,
            region_name=neutron_group.region_name)

    @mock.patch.object(socket, 'gethostname', return_value='fake_hostname')
    def test_get_hostname(self, mock_get_hostname):
        self.assertEqual('fake_hostname', utils.get_hostname())
        mock_get_hostname.assert_called_once()

    def test_get_dict_format_fixed_ips_from_kv_format(self):
        fake_fixed_ips_kv_format = \
            ['subnet_id=5083bda8-1b7c-4625-97f3-1d4c33bfeea8',
             'ip_address=192.168.1.2',
             'subnet_id=6607a230-f3eb-4937-b09f-9dd659211139',
             'ip_address=fdfa:8456:1afa:0:f816:3eff:fe67:885e']
        expected_dict_form = \
            [{'subnet_id': '5083bda8-1b7c-4625-97f3-1d4c33bfeea8',
              'ip_address': '192.168.1.2'},
             {'subnet_id': '6607a230-f3eb-4937-b09f-9dd659211139',
              'ip_address': 'fdfa:8456:1afa:0:f816:3eff:fe67:885e'}]
        fixed_ips = utils.get_dict_format_fixed_ips_from_kv_format(
                        fake_fixed_ips_kv_format)
        self.assertEqual(expected_dict_form, fixed_ips)

    def test_string_mappings(self):
        fake_mapping_list = {'fake_key': 'fake_value'}
        fake_result = '"{\'fake_key\': \'fake_value\'}"'
        self.assertEqual(fake_result, utils.string_mappings(fake_mapping_list))

    def test_get_random_string(self):
        fake_string_len = 20
        self.assertEqual(
             fake_string_len,
             len(utils.get_random_string(fake_string_len)))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_config import cfg
from oslotest import base

from kuryr.lib import config
from kuryr.lib import constants as const


class TestCase(base.BaseTestCase):
    """Test case base class for all unit tests."""

    def setUp(self):
        super(TestCase, self).setUp()
        CONF = cfg.CONF
        CONF.register_opts(config.core_opts)
        CONF.register_opts(config.binding_opts, group=config.binding_group)
        config.register_neutron_opts(CONF)

    @staticmethod
    def _get_fake_networks(neutron_network_id):
        fake_networks_response = {
            "networks": [{
                "status": "ACTIVE",
                "subnets": [],
                "name": "fake_network",
                "admin_state_up": True,
                "tenant_id": "9bacb3c5d39d41a79512987f338cf177",
                "router:external": False,
                "segments": [],
                "shared": False,
                "id": neutron_network_id
            }]
        }
        return fake_networks_response

    @staticmethod
    def _get_fake_subnets(docker_endpoint_id, neutron_network_id,
                          fake_neutron_subnet_v4_id,
                          fake_neutron_subnet_v6_id):
        # The following fake response is retrieved from the Neutron doc:
        #   https://docs.openstack.org/api-ref/network/v2/index.html#create-subnet  # noqa
        fake_subnet_response = {
            "subnets": [
                {
                    "name": '-'.join([docker_endpoint_id, '192.168.1.0']),
                    "network_id": neutron_network_id,
                    "tenant_id": "c1210485b2424d48804aad5d39c61b8f",
                    "allocation_pools": [{"start": "192.168.1.2",
                                          "end": "192.168.1.254"}],
                    "gateway_ip": "192.168.1.1",
                    "ip_version": 4,
                    "cidr": "192.168.1.0/24",
                    "id": fake_neutron_subnet_v4_id,
                    "enable_dhcp": True,
                    "subnetpool_id": ''
                },
                {
                    "name": '-'.join([docker_endpoint_id, 'fe80::']),
                    "network_id": neutron_network_id,
                    "tenant_id": "c1210485b2424d48804aad5d39c61b8f",
                    "allocation_pools": [{"start": "fe80::f816:3eff:fe20:57c4",
                                          "end": "fe80::ffff:ffff:ffff:ffff"}],
                    "gateway_ip": "fe80::f816:3eff:fe20:57c3",
                    "ip_version": 6,
                    "cidr": "fe80::/64",
                    "id": fake_neutron_subnet_v6_id,
                    "enable_dhcp": True,
                    "subnetpool_id": ''
                }
            ]
        }
        return fake_subnet_response

    @staticmethod
    def _get_fake_port(docker_endpoint_id, neutron_network_id,
                       neutron_port_id,
                       neutron_port_status=const.PORT_STATUS_DOWN,
                       neutron_subnet_v4_id=None,
                       neutron_subnet_v6_id=None,
                       neutron_subnet_v4_address="192.168.1.2",
                       neutron_subnet_v6_address="fe80::f816:3eff:fe20:57c4",
                       vif_details=None,
                       vif_type=None):
        # The following fake response is retrieved from the Neutron doc:
        #   https://docs.openstack.org/api-ref/network/v2/index.html#create-port  # noqa
        fake_port = {
            'port': {
                "status": neutron_port_status,
                "name": docker_endpoint_id + '-port',
                "allowed_address_pairs": [],
                "admin_state_up": True,
                "network_id": neutron_network_id,
                "tenant_id": "d6700c0c9ffa4f1cb322cd4a1f3906fa",
                "device_owner": "",
                "mac_address": "fa:16:3e:20:57:c3",
                "fixed_ips": [],
                "id": neutron_port_id,
                "security_groups": [],
                "device_id": "",
                "binding:vif_details": vif_details,
                "binding:vif_type": vif_type
            }
        }

        if neutron_subnet_v4_id:
            fake_port['port']['fixed_ips'].append({
                "subnet_id": neutron_subnet_v4_id,
                "ip_address": neutron_subnet_v4_address
            })
        if neutron_subnet_v6_id:
            fake_port['port']['fixed_ips'].append({
                "subnet_id": neutron_subnet_v6_id,
                "ip_address": neutron_subnet_v6_address
            })
        return fake_port
